-- ==============================================================
-- FILE : USER_AP1.VHD  -  MUST be modified by the USER
-- ==============================================================
--
-- This file is part of Example 1, for the HERON-IO2 module.
--
-- This file must be modified by the user.
--
-- ==============================================================
--
--  Date        : 08/02/2002
--  Author      : L. Bacquart - ALSE
--  Description : User code section
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     L. Bacquart    08-02-02    First Written
--  1.1     R. Williams    26-02-02    Changed names, added known
--                                     I/Os, modified IFD and OFD
--                                     implementation.
--  1.2     R. Williams    08-05-02    Syntax change made to all
--                                     instanciated components,
--                                     (changed '0' to GND)
--                                     for compatibility with
--                                     ModelSim5.5b.
--  2.0     R. Williams    10-05-02    Adapted for the IO2V2 from
--                                     IO2V1.
--  2.1     R. Williams    10-10-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--  2.4     R. Williams    08-06-06    Removed HEPC8 support.
--                                     Minor fix to HE_RD_6F logic.
--
-- ==============================================================



-- ==============================================================
--   Package : CONFIG
-- ==============================================================
--
-- This package allow users to set up the different clock domains
-- of the application (FIFO, ADC interfaces) in the top level.
-- Please refer to the HUNT documentation for more details.
--
-- ==============================================================

package CONFIG is

  -- FIFO CLOCK
  -- ==========

--   -----------------------------------------------------------------
--  | FCLK_G_DOMAIN |  HIGH_FCLK_G  |  HIGH_FCLK_RD  |  HIGH_FCLK_WR  |
--   -----------------------------------------------------------------
--  |     True      | True / False  |     n.a.       |     n.a.       |  single FIFO clock
--   -----------------------------------------------------------------
--  |     False     |     n.a.      |  True / False  |  True / False  |  two FIFO clocks
--   -----------------------------------------------------------------

  -- To use a single common FIFO clock, set FCLK_G_DOMAIN = TRUE
  constant FCLK_G_DOMAIN : boolean := TRUE;

  -- When read/write FIFO clock frequency > 60 MHz, set HIGH_FCLK_G = TRUE
  -- This constant is used only when FCLK_G_DOMAIN = TRUE
  constant HIGH_FCLK_G   : boolean := TRUE;

  -- When read FIFO clock frequency > 60 MHz, set HIGH_FCLK_RD = TRUE
  -- This constant is used only when FCLK_G_DOMAIN = FALSE
  constant HIGH_FCLK_RD  : boolean := FALSE;

  -- When write FIFO clock frequency > 60 MHz, set HIGH_FCLK_WR = TRUE
  -- This constant is used only when FCLK_G_DOMAIN = FALSE
  constant HIGH_FCLK_WR  : boolean := FALSE;

  -- ADC CLOCK
  -- =========

  -- To drive ADC sample clock A from a signal inside the FPGA set INTERNAL_SCLK_A = TRUE
  -- to use an external sample clock, set INTERNAL_SCLK_A = FALSE
  constant INTERNAL_SCLK_A : boolean := FALSE;

  -- To use a single common ADC clock for A & B channels, set SCLK_G_DOMAIN = TRUE
  constant SCLK_G_DOMAIN : boolean := TRUE;

end CONFIG;


-- ==============================================================
--   Module : USER_AP
-- ==============================================================

library IEEE;
  use IEEE.std_logic_1164.ALL;
  use IEEE.numeric_std.all;
  use WORK.CONFIG.all;

--pragma translate_off
library sim_lib;
  use sim_lib.all;

library UNISIM;
  use UNISIM.VCOMPONENTS.all;
--pragma translate_on

entity USER_AP is
  port (
    RESET           : in  std_logic;                     -- asynchronous reset active high
    CONFIG          : in  std_logic;                     -- System wide Config signal
-- CLOCK IO
    OSC0            : in  std_logic;                     -- Xtal OSC0
    OSC1            : in  std_logic;                     -- Xtal OSC1
    OSC2            : in  std_logic;                     -- Xtal OSC2
    OSC3            : in  std_logic;                     -- Xtal OSC3 factory default fitted with 100Mhz
    CLKIN0          : in  std_logic;                     -- User clock input (unbuffered) 
    CLKIN1          : in  std_logic;                     -- User clock input (unbuffered)
    CLKI2           : in  std_logic;                     -- User clock input (buffered)
    CLKI3           : in  std_logic;                     -- User clock input (buffered)
    CLKOUT          : out std_logic;                     -- User clock output
    QTTL            : in  std_logic;
    DTTL            : out std_logic;
    PECL_CLK        : in  std_logic;
-- SERIAL IO
    R1OUT           : in  std_logic;
    R2OUT           : in  std_logic;
    T1IN            : out std_logic;
    T2IN            : out std_logic;
    RS485_232       : out std_logic;
    HDPLX           : out std_logic;
    FAST            : out std_logic;
-- MISCELLANEOUS IO
    UDPRES          : out std_logic;                     -- reset the carrier (high if NOT used)
    LED             : out std_logic_vector(4 downto 0);  -- LEDs
    ADDR_FLAGSEL    : in  std_logic;
    BOOTEN          : out std_logic;
    CID             : in  std_logic_vector(3 downto 0);
    MID             : in  std_logic_vector(3 downto 0);
-- UMIs
    UMI_IN          : in  std_logic_vector(3 downto 0);
    UMI_OUT         : out std_logic_vector(3 downto 0);
    UMI_EN          : out std_logic_vector(3 downto 0);
-- DIGITAL IO
    DIO_IN          : in  std_logic_vector(7 downto 0);
    DIO_OUT         : out std_logic_vector(7 downto 0);
    DIO_EN          : out std_logic_vector(7 downto 0);
-- READ FIFO CLK
    FCLK_RD         : in  std_logic;                     -- FIFO_IN clock to be used in this module (only when FCLK_G_DOMAIN=FALSE)
    SRC_FCLK_RD     : out std_logic;                     -- FIFO_IN clock source for the top level  (only when FCLK_G_DOMAIN=FALSE)
-- WRITE FIFO CLK
    FCLK_WR         : in  std_logic;                     -- FIFO_OUT clock to be used in this module (only when FCLK_G_DOMAIN=FALSE)
    SRC_FCLK_WR     : out std_logic;                     -- FIFO_OUT clock source for the top level  (only when FCLK_G_DOMAIN=FALSE)
-- COMMON FIFO CLK
    FCLK_G          : in  std_logic;                     -- Common FIFO clock to be used in this module (only when FCLK_G_DOMAIN=TRUE)
    SRC_FCLK_G      : out std_logic;                     -- Common FIFO clock source for the top level  (only when FCLK_G_DOMAIN=TRUE)
-- FIFO IN / HE_RD_6F interface
    INFIFO_READ_REQ : out std_logic_vector(5 downto 0);  -- FIFO_IN Read Request
    INFIFO_DVALID   : in  std_logic_vector(5 downto 0);  -- FIFO_IN Data Valid
    INFIFO_SINGLE   : in  std_logic_vector(5 downto 0);  -- FIFO_IN Single data item available
    INFIFO_BURST    : in  std_logic_vector(5 downto 0);  -- FIFO_IN Block of data available
    INFIFO0_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 0
    INFIFO1_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 1
    INFIFO2_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 2
    INFIFO3_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 3
    INFIFO4_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 4
    INFIFO5_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 5
-- FIFO OUT / HE_WR_6F interface
    OUTFIFO_READY   : in  std_logic_vector(5 downto 0);  -- FIFO_OUT Ready Flag
    OUTFIFO_WRITE   : out std_logic_vector(5 downto 0);  -- FIFO_OUT Write Command
    OUTFIFO_D       : out std_logic_vector(31 downto 0); -- FIFO_OUT Data
-- SERIAL BUS / HE_USER Message interface
    MSG_CLK         : out std_logic;                     -- HSB clock source for the top level
    MSG_DIN         : in  std_logic_vector(7 downto 0);  -- data received from HSB
    MSG_ADDR        : in  std_logic_vector(7 downto 0);  -- HSB address
    MSG_AEN         : in  std_logic;                     -- address enable
    MSG_WEN         : in  std_logic;                     -- write access
    MSG_REN         : in  std_logic;                     -- read access
    MSG_DONE        : in  std_logic;                     -- message successfully transmitted
    MSG_COUNT       : in  std_logic;                     -- user's counter enable
    MSG_CLEAR       : in  std_logic;                     -- user's counter clear
    MSG_READY       : out std_logic;                     -- to acknowledge an access
    MSG_SEND_MSG    : out std_logic;                     -- message send command
    MSG_CE          : out std_logic;                     -- to control speed operation
    MSG_DOUT        : out std_logic_vector(7 downto 0);  -- data sent to HSB
    MSG_SEND_ID     : out std_logic;                     -- ID information send command
    MSG_LAST_BYTE   : out std_logic;                     -- to indicate when the last byte is presented
-- ADC_A SCLK
    SCLK_A          : in  std_logic;                     -- ADC A clock to be used in this module (only when SCLK_G_DOMAIN=FALSE)
    SRC_SCLK_A      : out std_logic;                     -- ADC A clock source for the top level  (only when SCLK_G_DOMAIN=FALSE)
-- ADC_B SCLK
    SCLK_B          : in  std_logic;                     -- ADC B clock to be used in this module (only when SCLK_G_DOMAIN=FALSE)
    SRC_SCLK_B      : out std_logic;                     -- ADC B clock source for the top level  (only when SCLK_G_DOMAIN=FALSE)
-- GLOBAL ADC SCLK
    SCLK_G          : in  std_logic;                     -- Common ADC clock to be used in this module (only when SCLK_G_DOMAIN=TRUE)
    SRC_SCLK_G      : out std_logic;                     -- Common ADC clock source for the top level  (only when SCLK_G_DOMAIN=TRUE)
-- HE_ADC_A
    ADC_A           : in  std_logic_vector(11 downto 0); -- ADC_A data
    OTR_A           : in  std_logic;                     -- ADC_A over range flag
-- HE_ADC_B
    ADC_B           : in  std_logic_vector(11 downto 0); -- ADC_B data
    OTR_B           : in  std_logic;                     -- ADC_B over range flag
-- HE_DAC_A
    SCLK_DAC_A      : out std_logic;                     -- DAC_A clock source for the top level
    DAC_A           : out std_logic_vector(13 downto 0); -- DAC_A data
-- HE_DAC_B
    SCLK_DAC_B      : out std_logic;                     -- DAC_B clock source for the top level
    DAC_B           : out std_logic_vector(13 downto 0)  -- DAC_B data
  );
end USER_AP;


architecture EXAMPLE1 of USER_AP is

  component HSB1
    port (
      RST         : in  std_logic;
      CLK         : in  std_logic;
      DIN         : in  std_logic_vector(7 downto 0);
      ADDR        : in  std_logic_vector(7 downto 0);
      AEN         : in  std_logic;
      WEN         : in  std_logic;
      REN         : in  std_logic;
      DONE        : in  std_logic;
      COUNT       : in  std_logic;
      CLEAR       : in  std_logic;
      READY       : out std_logic;
      SEND_MSG    : out std_logic;
      CE          : out std_logic;
      DOUT        : out std_logic_vector(7 downto 0);
      SEND_ID     : out std_logic;
      LAST_BYTE   : out std_logic;
      --
      INFIFO_SEL  : out std_logic_vector (5 downto 0);
      OUTFIFO_SEL : out std_logic_vector (5 downto 0)
    );
  end component;

  component fifo15x32
    port (
      din          : in  std_logic_vector(31 downto 0);
      wr_en        : in  std_logic;
      wr_clk       : in  std_logic;
      rd_en        : in  std_logic;
      rd_clk       : in  std_logic;
      ainit        : in  std_logic;
      dout         : out std_logic_vector(31 downto 0);
      full         : out std_logic;
      empty        : out std_logic;
      almost_full  : out std_logic;
      almost_empty : out std_logic
    );
  end component;

  signal MCLK        : std_logic;  -- Main clock
  signal LEDcount    : unsigned(25 downto 0);
  signal MUX_SEL     : std_logic_vector(2 downto 0);
  signal INFIFO_SEL  : std_logic_vector(5 downto 0);
  signal OUTFIFO_SEL : std_logic_vector(5 downto 0);
  --
  signal FIFO_A      : std_logic_vector(31 downto 0);
  signal FIFO_B      : std_logic_vector(31 downto 0);
  signal FIFO_C      : std_logic_vector(31 downto 0);
  signal FIFO_D      : std_logic_vector(31 downto 0);
  signal FIFO_E      : std_logic_vector(31 downto 0);
  signal FIFO_F      : std_logic_vector(31 downto 0);
  signal FIFO_G      : std_logic_vector(31 downto 0);
  signal FIFO_AF     : std_logic_vector(5 downto 0);
  signal FIFO_EF     : std_logic_vector(5 downto 0);
  signal FIFO_RD     : std_logic_vector(5 downto 0);
  -- Control signals for transfer from FIFOs A-F to FIFO G
  signal NOT_EMPTY_A : std_logic;
  signal NOT_EMPTY_B : std_logic;
  signal NOT_EMPTY_C : std_logic;
  signal WR_EN       : std_logic;
  signal AF          : std_logic;
  signal AF_v        : std_logic_vector(5 downto 0);
  -- Control signals for transfer from FIFO G to HERON FIFO-Write Interface
  signal READY_A     : std_logic;
  signal READY_B     : std_logic;
  signal READY_C     : std_logic;
  signal RD_EN       : std_logic;
  signal EMPTY       : std_logic;
  signal EMPTY_v     : std_logic_vector(5 downto 0);
  signal WRITE       : std_logic_vector(5 downto 0);

begin


  -- ==========================================================
  -- ==========================================================
  --       <<<<<<<   INSERT YOUR CODE HERE    >>>>>>>>>
  -- ==========================================================
  -- ==========================================================


  -------------------------------------------------------------
  -- OSC3 clock

  MCLK <= OSC3;

  -------------------------------------------------------------
  -- Drive the UDP-RESET high (inactive)

  UDPRES <= '1';

  -------------------------------------------------------------
  -- Drive the BOOT ENABLE signal high (inactive)

  BOOTEN <= '1';

  -------------------------------------------------------------
  -- Drive unused signals

  DIO_OUT   <= (others=>'0');
  DIO_EN    <= (others=>'1');

  UMI_OUT   <= (others=>'0');
  UMI_EN    <= (others=>'1');

  CLKOUT    <= '0';

  DTTL      <= '0';
  T1IN      <= '0';
  T2IN      <= '0';
  RS485_232 <= '0';
  HDPLX     <= '0';
  FAST      <= '0';

  -------------------------------------------------------------
  -- FIFOs

  SRC_FCLK_RD <= '0';   -- clock source for READ FIFO
  SRC_FCLK_WR <= '0';   -- clock source for WRITE FIFO
  SRC_FCLK_G  <= MCLK;  -- global clock source for both FIFOs

  INFIFO_READ_REQ <= NOT FIFO_AF;

  iFIFO0 : fifo15x32
    port map (
      din          => INFIFO0_D,
      wr_en        => INFIFO_DVALID(0),
      wr_clk       => FCLK_G,
      rd_en        => FIFO_RD(0),
      rd_clk       => FCLK_G,
      ainit        => RESET,
      dout         => FIFO_A,
      full         => open,
      empty        => FIFO_EF(0),
      almost_full  => FIFO_AF(0),
      almost_empty => open );

  iFIFO1 : fifo15x32
    port map (
      din          => INFIFO1_D,
      wr_en        => INFIFO_DVALID(1),
      wr_clk       => FCLK_G,
      rd_en        => FIFO_RD(1),
      rd_clk       => FCLK_G,
      ainit        => RESET,
      dout         => FIFO_B,
      full         => open,
      empty        => FIFO_EF(1),
      almost_full  => FIFO_AF(1),
      almost_empty => open );

  iFIFO2 : fifo15x32
    port map (
      din          => INFIFO2_D,
      wr_en        => INFIFO_DVALID(2),
      wr_clk       => FCLK_G,
      rd_en        => FIFO_RD(2),
      rd_clk       => FCLK_G,
      ainit        => RESET,
      dout         => FIFO_C,
      full         => open,
      empty        => FIFO_EF(2),
      almost_full  => FIFO_AF(2),
      almost_empty => open );

  iFIFO3 : fifo15x32
    port map (
      din          => INFIFO3_D,
      wr_en        => INFIFO_DVALID(3),
      wr_clk       => FCLK_G,
      rd_en        => FIFO_RD(3),
      rd_clk       => FCLK_G,
      ainit        => RESET,
      dout         => FIFO_D,
      full         => open,
      empty        => FIFO_EF(3),
      almost_full  => FIFO_AF(3),
      almost_empty => open );

  iFIFO4 : fifo15x32
    port map (
      din          => INFIFO4_D,
      wr_en        => INFIFO_DVALID(4),
      wr_clk       => FCLK_G,
      rd_en        => FIFO_RD(4),
      rd_clk       => FCLK_G,
      ainit        => RESET,
      dout         => FIFO_E,
      full         => open,
      empty        => FIFO_EF(4),
      almost_full  => FIFO_AF(4),
      almost_empty => open );

  iFIFO5 : fifo15x32
    port map (
      din          => INFIFO5_D,
      wr_en        => INFIFO_DVALID(5),
      wr_clk       => FCLK_G,
      rd_en        => FIFO_RD(5),
      rd_clk       => FCLK_G,
      ainit        => RESET,
      dout         => FIFO_F,
      full         => open,
      empty        => FIFO_EF(5),
      almost_full  => FIFO_AF(5),
      almost_empty => open );

  process(FCLK_G)
  begin
    if rising_edge(FCLK_G) then
      MUX_SEL(0) <= INFIFO_SEL(1) OR INFIFO_SEL(3) OR INFIFO_SEL(5);
      MUX_SEL(1) <= INFIFO_SEL(2) OR INFIFO_SEL(3);
      MUX_SEL(2) <= INFIFO_SEL(4) OR INFIFO_SEL(5);
    end if;
  end process;

  process (MUX_SEL, FIFO_A, FIFO_B, FIFO_C, FIFO_D, FIFO_E, FIFO_F)
  begin
    case MUX_SEL is
      when "000"  => FIFO_G <= FIFO_A;
      when "001"  => FIFO_G <= FIFO_B;
      when "010"  => FIFO_G <= FIFO_C;
      when "011"  => FIFO_G <= FIFO_D;
      when "100"  => FIFO_G <= FIFO_E;
      when "101"  => FIFO_G <= FIFO_F;
      when others => FIFO_G <= (others=>'0');
    end case;
  end process;

  AF_v <= AF & AF & AF & AF & AF & AF;

  FIFO_RD <= (NOT FIFO_EF) AND (NOT AF_v) AND INFIFO_SEL;

  NOT_EMPTY_A <= ((NOT FIFO_EF(0)) AND INFIFO_SEL(0)) OR
                 ((NOT FIFO_EF(1)) AND INFIFO_SEL(1));

  NOT_EMPTY_B <= ((NOT FIFO_EF(2)) AND INFIFO_SEL(2)) OR
                 ((NOT FIFO_EF(3)) AND INFIFO_SEL(3));

  NOT_EMPTY_C <= ((NOT FIFO_EF(4)) AND INFIFO_SEL(4)) OR
                 ((NOT FIFO_EF(5)) AND INFIFO_SEL(5));

  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      WR_EN  <= '0';
    elsif rising_edge(FCLK_G) then
      WR_EN <= '0';
      if (NOT_EMPTY_A='1' or NOT_EMPTY_B='1' or NOT_EMPTY_C='1') and AF='0' then
        WR_EN <= '1';
      end if;
    end if;
  end process;

  iFIFO6 : fifo15x32
    port map (
      din          => FIFO_G,
      wr_en        => WR_EN,
      wr_clk       => FCLK_G,
      rd_en        => RD_EN,
      rd_clk       => FCLK_G,
      ainit        => RESET,
      dout         => OUTFIFO_D,
      full         => open,
      empty        => EMPTY,
      almost_full  => AF,
      almost_empty => open );

  READY_A <= (OUTFIFO_READY(0) AND OUTFIFO_SEL(0)) OR
             (OUTFIFO_READY(1) AND OUTFIFO_SEL(1));

  READY_B <= (OUTFIFO_READY(2) AND OUTFIFO_SEL(2)) OR
             (OUTFIFO_READY(3) AND OUTFIFO_SEL(3));

  READY_C <= (OUTFIFO_READY(4) AND OUTFIFO_SEL(4)) OR
             (OUTFIFO_READY(5) AND OUTFIFO_SEL(5));

  RD_EN <= (READY_A OR READY_B OR READY_C) AND (NOT EMPTY);

  EMPTY_v <= EMPTY & EMPTY & EMPTY & EMPTY & EMPTY & EMPTY;

  WRITE <= (NOT EMPTY_v) AND OUTFIFO_READY AND OUTFIFO_SEL;

  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      OUTFIFO_WRITE <= (others=>'0');
    elsif rising_edge(FCLK_G) then
      OUTFIFO_WRITE <= WRITE;
    end if;
  end process;

  -------------------------------------------------------------
  -- Serial Bus

  MSG_CLK <= OSC3;  -- clock for Message Interface

  iHSB1 : HSB1
    port map (
      RST         => RESET,
      CLK         => OSC3,
      DIN         => MSG_DIN,
      ADDR        => MSG_ADDR,
      AEN         => MSG_AEN,
      WEN         => MSG_WEN,
      REN         => MSG_REN,
      DONE        => MSG_DONE,
      COUNT       => MSG_COUNT,
      CLEAR       => MSG_CLEAR,
      READY       => MSG_READY,
      SEND_MSG    => MSG_SEND_MSG,
      CE          => MSG_CE,
      DOUT        => MSG_DOUT,
      SEND_ID     => MSG_SEND_ID,
      LAST_BYTE   => MSG_LAST_BYTE,
      INFIFO_SEL  => INFIFO_SEL,
      OUTFIFO_SEL => OUTFIFO_SEL );

  -------------------------------------------------------------
  -- ADC (not used in this example)

  SRC_SCLK_A <= '0';  -- clock source for ADC_A
  SRC_SCLK_B <= '0';  -- clock source for ADC_B
  SRC_SCLK_G <= '0';  -- global clock source for both ADCs

  -------------------------------------------------------------
  -- DAC (not used in this example)

  SCLK_DAC_A <= '0';  -- clock of DAC_A
  SCLK_DAC_B <= '0';  -- clock of DAC_B

  DAC_A <= (others=>'0');
  DAC_B <= (others=>'0');

  -------------------------------------------------------------
  -- Led Flasher : dividing F_CLKG clock by 2**25

  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      LEDcount <= (others=>'0');
    elsif rising_edge(FCLK_G) then
      LEDcount <= LEDcount + 1;
    end if;
  end process;

  LED(4) <= LEDcount(24);

  -- Drive LEDs high (inactive)
  LED(3 downto 0) <= (others=>'1');

end EXAMPLE1;

