-- ==============================================================
-- FILE : CTR_ADC.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of Example 2, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : CTR_ADC
--  Date        : 07/02/2002
--  Author      : L. Bacquart - ALSE
--  Description : ADC data flow Control
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     L. Bacquart    07-02-02    First Written
--  1.1     R. Williams    26-02-02    Changed names, added known
--                                     I/Os, modified IFD and OFD
--                                     implementation.
--  2.0     R. Williams    10-05-02    Adapted for the IO2V2 from
--                                     IO2V1.
--  2.1     R. Williams    10-10-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity CTR_ADC is
  port (
    RST           : in  std_logic;
    SCLK          : in  std_logic;
    FCLK          : in  std_logic;
    UPDATE        : in  std_logic;
    SAMPLE        : in  std_logic_vector(8 downto 0);
    FF_FULL       : in  std_logic;
    FF_EMPTY      : in  std_logic;
    UPD_DONE      : out std_logic;
    FF_WEN        : out std_logic;
    FF_REN        : out std_logic;
    OUTFIFO_SEL   : in  std_logic_vector(5 downto 0);
    OUTFIFO_READY : in  std_logic_vector(5 downto 0);
    OUTFIFO_WRITE : out std_logic_vector(5 downto 0)
  );
end CTR_ADC;


architecture RTL of CTR_ADC is

  signal UPD_R  : std_logic;
  signal UPD_RR : std_logic;
  signal State  : std_logic;
  signal count  : unsigned(8 downto 0);
  signal GO     : std_logic;
  signal READY  : std_logic;
  signal WRITE  : std_logic;

begin

  -- GO trigger
  -- ==========
  process(RST, SCLK)
  begin
    if RST='1' then
      UPD_R <= '0';
      UPD_RR <= '0';
    elsif rising_edge(SCLK) then
      UPD_R <= UPDATE;
      UPD_RR <= UPD_R;
    end if;
  end process;

  UPD_DONE <= UPD_RR;

  process(RST, SCLK)
  begin
    if RST='1' then
      State <= '0';
      count <= (others=>'0');
      GO <= '0';
    elsif rising_edge(SCLK) then
      if State='0' then
        GO <= '0';

        if UPD_RR='1' then
          State <= '1';
          count <= unsigned(SAMPLE);
        end if;
      else
        GO <= '1';

        if count=1 then
          State <= '0';
        end if;
        count <= count - 1;
      end if;
    end if;
  end process;

  -- Control lines
  -- =============
  FF_WEN <= (NOT FF_FULL) AND GO;

  FF_REN <= (NOT FF_EMPTY) AND READY;

  READY <= (OUTFIFO_READY(0) AND OUTFIFO_SEL(0)) OR
           (OUTFIFO_READY(1) AND OUTFIFO_SEL(1)) OR
           (OUTFIFO_READY(2) AND OUTFIFO_SEL(2)) OR
           (OUTFIFO_READY(3) AND OUTFIFO_SEL(3)) OR
           (OUTFIFO_READY(4) AND OUTFIFO_SEL(4)) OR
           (OUTFIFO_READY(5) AND OUTFIFO_SEL(5));

  process (RST, FCLK)
  begin
     if RST ='1' then
      WRITE <= '0';
    elsif rising_edge(FCLK) then
      WRITE <= '0';
      if FF_EMPTY='0' and READY='1' then
        WRITE <= '1';
      end if;
    end if;
  end process;

  OUTFIFO_WRITE(0) <= WRITE AND OUTFIFO_SEL(0);
  OUTFIFO_WRITE(1) <= WRITE AND OUTFIFO_SEL(1);
  OUTFIFO_WRITE(2) <= WRITE AND OUTFIFO_SEL(2);
  OUTFIFO_WRITE(3) <= WRITE AND OUTFIFO_SEL(3);
  OUTFIFO_WRITE(4) <= WRITE AND OUTFIFO_SEL(4);
  OUTFIFO_WRITE(5) <= WRITE AND OUTFIFO_SEL(5);

end RTL;
