-- ==============================================================
-- FILE : CTR_DAC.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of Example 2, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : CTR_DAC
--  Date        : 07/02/2002
--  Author      : L. Bacquart - ALSE
--  Description : DAC data flow Control
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     L. Bacquart    07-02-02    First Written
--  1.1     R. Williams    26-02-02    Changed names, added known
--                                     I/Os, modified IFD and OFD
--                                     implementation.
--  2.0     R. Williams    10-05-02    Adapted for the IO2V2 from
--                                     IO2V1.
--  2.1     R. Williams    10-10-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity CTR_DAC is
  port (
    RST             : in  std_logic;
    FCLK            : in  std_logic;
    SCLK            : in  std_logic;
    READ_EN         : in  std_logic;
    WRITE_A         : in  std_logic;
    RESET_A         : in  std_logic;
    WRITE_B         : in  std_logic;
    RESET_B         : in  std_logic;
    INFIFO_SEL      : in  std_logic_vector(5 downto 0);
    INFIFO_DVALID   : in  std_logic_vector(5 downto 0);
    INFIFO_READ_REQ : out std_logic_vector(5 downto 0);
    WEN_A           : out std_logic;
    WR_ADDR_A       : out std_logic_vector(8 downto 0);
    WEN_B           : out std_logic;
    WR_ADDR_B       : out std_logic_vector(8 downto 0);
    RD_ADDR         : out std_logic_vector(8 downto 0)
  );
end CTR_DAC;


architecture RTL of CTR_DAC is

  signal READ_REQ   : std_logic;
  signal DVALID     : std_logic;
  signal WRITE      : std_logic;
  signal WR_A       : std_logic;
  signal WR_B       : std_logic;
  signal CLEAR_A    : std_logic;
  signal CLEAR_B    : std_logic;
  signal RDcounter  : unsigned(8 downto 0);
  signal WRAcounter : unsigned(8 downto 0);
  signal WRBcounter : unsigned(8 downto 0);

begin

  READ_REQ <= '1';

  INFIFO_READ_REQ(0) <= READ_REQ AND INFIFO_SEL(0);
  INFIFO_READ_REQ(1) <= READ_REQ AND INFIFO_SEL(1);
  INFIFO_READ_REQ(2) <= READ_REQ AND INFIFO_SEL(2);
  INFIFO_READ_REQ(3) <= READ_REQ AND INFIFO_SEL(3);
  INFIFO_READ_REQ(4) <= READ_REQ AND INFIFO_SEL(4);
  INFIFO_READ_REQ(5) <= READ_REQ AND INFIFO_SEL(5);

  DVALID <= (INFIFO_DVALID(0) AND INFIFO_SEL(0)) OR
            (INFIFO_DVALID(1) AND INFIFO_SEL(1)) OR
            (INFIFO_DVALID(2) AND INFIFO_SEL(2)) OR
            (INFIFO_DVALID(3) AND INFIFO_SEL(3)) OR
            (INFIFO_DVALID(4) AND INFIFO_SEL(4)) OR
            (INFIFO_DVALID(5) AND INFIFO_SEL(5));

  WR_A <= DVALID AND WRITE_A;
  WR_B <= DVALID AND WRITE_B;

  WEN_A <= WR_A;
  WEN_B <= WR_B;

  -- READ address
  process(RST, SCLK, READ_EN)
  begin
    if RST='1' then
      RDcounter <= (others=>'0');
    elsif (rising_edge(SCLK) AND (READ_EN = '1')) then
      RDcounter <= RDcounter + 1;
    end if;
  end process;

  RD_ADDR <= std_logic_vector(RDcounter);

  -- WRITE address for channel A
  CLEAR_A <= (WRITE AND RESET_A) OR RST;

  process(CLEAR_A, FCLK)
  begin
    if CLEAR_A='1' then
      WRAcounter <= (others=>'0');
    elsif rising_edge(FCLK) then
      if WR_A='1' then
        WRAcounter <= WRAcounter + 1;
      end if;
    end if;
  end process;

  WR_ADDR_A <= std_logic_vector(WRAcounter);

  -- WRITE address for channel B
  CLEAR_B <= (WRITE AND RESET_B) OR RST;

  process(CLEAR_B, FCLK)
  begin
    if CLEAR_B='1' then
      WRBcounter <= (others=>'0');
    elsif rising_edge(FCLK) then
      if WR_B='1' then
        WRBcounter <= WRBcounter + 1;
      end if;
    end if;
  end process;

  WR_ADDR_B <= std_logic_vector(WRBcounter);

end RTL;
