-- ==============================================================
-- FILE : CTR_ADC.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of FFT_Example, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : CTR_ADC
--  Date        : 07/02/2002
--  Author      : L. Bacquart - ALSE
--  Description : ADC data flow Control
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     L. Bacquart    07-02-02    First Written
--  1.1     R. Williams    26-02-02    Changed names, added known
--                                     I/Os, modified IFD and OFD
--                                     implementation.
--  2.0     R. Williams    10-05-02    Adapted for the IO2V2 from
--                                     IO2V1.
--  2.1     R. Williams    10-10-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--  3.0     T. Hollis      11-06-03    Modified for fft_example
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity CTR_ADC is
  port (
    RST           : in  std_logic;
    SCLK          : in  std_logic;
    FCLK          : in  std_logic;
    UPDATE        : in  std_logic;
    SAMPLE        : in  std_logic_vector(9 downto 0);
    FF_FULL       : in  std_logic;
    FF_EMPTY      : in  std_logic;
    UPD_DONE      : out std_logic;
    FF_WEN        : out std_logic;
    FF_REN        : out std_logic;
    OUTFIFO_SEL   : in  std_logic_vector(5 downto 0);
    OUTFIFO_READY : in  std_logic_vector(5 downto 0);
    OUTFIFO_WRITE : out std_logic_vector(5 downto 0);
    SYNC          : in  std_logic
  );
end CTR_ADC;


architecture RTL of CTR_ADC is

  type STATE_TYPE is (S1, S2, S3, S4);
  attribute ENUM_ENCODING: STRING;
  attribute ENUM_ENCODING of STATE_TYPE: type is "0001 0010 0100 1000";
  signal CS, NS: STATE_TYPE;

  signal UPD_R  : std_logic;
  signal UPD_RR : std_logic;
  signal State  : std_logic;
  signal count  : unsigned(9 downto 0);
  signal GO     : std_logic;
  signal READY  : std_logic;
  signal WRITE  : std_logic;

  signal COUNT_LD  : std_logic;
  signal COUNT_EN  : std_logic;
  signal DONE      : std_logic;

begin

  -- GO trigger
  -- ==========
  process(RST, SCLK)
  begin
    if RST='1' then
      UPD_R <= '0';
      UPD_RR <= '0';
    elsif rising_edge(SCLK) then
      UPD_R <= UPDATE;
      UPD_RR <= UPD_R;
    end if;
  end process;

  UPD_DONE <= UPD_RR;

  -- DATA O/P STATE MACHINE 
SYNC_PROC: process (SCLK, RST)
    begin
   if (RST='1') then
            CS <= S1;
       elsif rising_edge(SCLK) then
            CS <= NS;
       end if;
    end process;
 
COMB_PROC: process (CS, UPD_RR, SYNC, DONE)
    begin
       case CS is
            when S1 =>   -- WAIT FOR DATA REQUEST
        GO       <= '0';
        COUNT_LD <= '0';
        COUNT_EN <= '0';
        if (UPD_RR = '0') then NS <= S1;
        else                   NS <= S2;
        end if;

            when S2 =>   -- WAIT FOR NEXT FFT O/P BLOCK
        GO       <= '0';
        COUNT_LD <= '0';
        COUNT_EN <= '0';
        if (SYNC = '0')   then NS <= S2;
        else                   NS <= S3;
        end if;

            when S3 =>   -- LOAD COUNT
        GO       <= '0';
        COUNT_LD <= '1';
        COUNT_EN <= '0';
                               NS <= S4;

            when S4 =>   -- TRANSFER DATA
        GO       <= '1';
        COUNT_LD <= '0';
        COUNT_EN <= '1';
        if (DONE = '1')  then  NS <= S1;
        else                   NS <= S4;
        end if;

      end case;
 end process;

   process(SCLK, RST)
  begin
    if RST='1' then
      count <= (others => '0');
    elsif rising_edge(SCLK)  then

        if    (COUNT_LD = '1') then count <= unsigned(SAMPLE);
        elsif (COUNT_EN = '1') then count <= count - 1;
	   else                        count <= count;
        end if;

        if count = 2 then
           DONE <= '1';
        else
           DONE <= '0';
        end if;

    end if;
  end process;


  -- Control lines
  -- =============
  FF_WEN <= (NOT FF_FULL) AND GO;

  FF_REN <= (NOT FF_EMPTY) AND READY;

  READY <= (OUTFIFO_READY(0) AND OUTFIFO_SEL(0)) OR
           (OUTFIFO_READY(1) AND OUTFIFO_SEL(1)) OR
           (OUTFIFO_READY(2) AND OUTFIFO_SEL(2)) OR
           (OUTFIFO_READY(3) AND OUTFIFO_SEL(3)) OR
           (OUTFIFO_READY(4) AND OUTFIFO_SEL(4)) OR
           (OUTFIFO_READY(5) AND OUTFIFO_SEL(5));

  process (RST, FCLK)
  begin
     if RST ='1' then
      WRITE <= '0';
    elsif rising_edge(FCLK) then
      WRITE <= '0';
      if FF_EMPTY='0' and READY='1' then
        WRITE <= '1';
      end if;
    end if;
  end process;

  OUTFIFO_WRITE(0) <= WRITE AND OUTFIFO_SEL(0);
  OUTFIFO_WRITE(1) <= WRITE AND OUTFIFO_SEL(1);
  OUTFIFO_WRITE(2) <= WRITE AND OUTFIFO_SEL(2);
  OUTFIFO_WRITE(3) <= WRITE AND OUTFIFO_SEL(3);
  OUTFIFO_WRITE(4) <= WRITE AND OUTFIFO_SEL(4);
  OUTFIFO_WRITE(5) <= WRITE AND OUTFIFO_SEL(5);

end RTL;
