-- ==============================================================
-- FILE : SIG_GEN.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of DDC Example, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : SIG_GEN
--  Date        : 07/02/2002
--  Author      : L. Bacquart - ALSE
--  Description : DAC data flow Control
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0       T.Hollis     20-03-03    First Written

--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity SIG_GEN is
  port (
      RST             : in  std_logic;
      FCLK            : in  std_logic;
      SCLK            : in  std_logic;
      --Heron read interface
      INFIFO_SEL      : in  std_logic_vector(5 downto 0);
      INFIFO_DVALID   : in  std_logic_vector(5 downto 0);
      INFIFO_READ_REQ : out std_logic_vector(5 downto 0);
      INFIFO0_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 0
      INFIFO1_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 1
      INFIFO2_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 2
      INFIFO3_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 3
      INFIFO4_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 4
      INFIFO5_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 5
      -- HE_DAC_A
      SCLK_DAC_A      : out std_logic;                     -- DAC_A clock source for the top level
      DAC_A           : out std_logic_vector(13 downto 0) -- DAC_A data 
  );
end SIG_GEN;


architecture RTL of SIG_GEN is

  component CTR_DAC
    port (
      RST             : in  std_logic;
      FCLK            : in  std_logic;
      SCLK            : in  std_logic;
      READ_EN         : in  std_logic;
      WRITE_A         : in  std_logic;
      RESET_A         : in  std_logic;
      WRITE_B         : in  std_logic;
      RESET_B         : in  std_logic;
      INFIFO_SEL      : in  std_logic_vector(5 downto 0);
      INFIFO_DVALID   : in  std_logic_vector(5 downto 0);
      INFIFO_READ_REQ : out std_logic_vector(5 downto 0);
      WEN_A           : out std_logic;
      WR_ADDR_A       : out std_logic_vector(8 downto 0);
      WEN_B           : out std_logic;
      WR_ADDR_B       : out std_logic_vector(8 downto 0);
      RD_ADDR         : out std_logic_vector(8 downto 0)
    );
  end component;

    component DPRAM
    port (
      RST     : in  std_logic;
      WRCLK   : in  std_logic;
      WEN     : in  std_logic;
      WR_ADDR : in  std_logic_vector(8 downto 0);
      DIN     : in  std_logic_vector(15 downto 0);
      --
      RDCLK   : in  std_logic;
      REN     : in  std_logic;
      RD_ADDR : in  std_logic_vector(8 downto 0);
      DOUT    : out std_logic_vector(15 downto 0)
    );
  end component;

     component ddc_add
   port (
     A     : in   std_logic_VECTOR(13 downto 0);
     B     : in   std_logic_VECTOR(13 downto 0);
     Q     : out  std_logic_VECTOR(13 downto 0);
     CLK   : in   std_logic
    );
   end component;



  signal DATA_IN     : std_logic_vector(31 downto 0);

  signal MUX_SEL     : std_logic_vector(2 downto 0);
  signal UPDATE      : std_logic;
  signal SAMPLE      : std_logic_vector(8 downto 0);

  signal WEN_A       : std_logic;
  signal WR_ADDR_A   : std_logic_vector(8 downto 0);
  signal WEN_B       : std_logic;
  signal WR_ADDR_B   : std_logic_vector(8 downto 0);
  signal RD_ADDR     : std_logic_vector(8 downto 0);
  signal DATA_A      : std_logic_vector(15 downto 0);
  signal DATA_B      : std_logic_vector(15 downto 0);

  signal DAC_DIV     : std_logic;
  signal DATA_AB_RDY    : std_logic;
  signal DAC_clk     : std_logic;
  signal DACcounter  : unsigned(5 downto 0);
  signal DACcount    : std_logic_vector(5 downto 0);

  signal ADD_OP     : std_logic_vector(13 downto 0);
  signal ADD_OP_RDY : std_logic;


begin

  -- generate clock for DPRAM read 
   process(RST, SCLK)
  begin
   if RST='1' then
      DACcounter <= (others=>'0');
    elsif rising_edge(SCLK) then
      DACcounter <= DACcounter + 1;
    end if;
  end process;

   DACcount <=std_logic_vector(DACcounter);

   process(RST, SCLK)
  begin
    if RST='1' then
      DAC_DIV <= '0';
    elsif rising_edge(SCLK) then
      if (DACcount = "000000") then
         DAC_DIV <= '1';
      else
         DAC_DIV <= '0';
      end if;
    end if;
  end process;

  process(FCLK)
  begin
    if rising_edge(FCLK) then
      MUX_SEL(0) <= INFIFO_SEL(1) OR INFIFO_SEL(3) OR INFIFO_SEL(5);
      MUX_SEL(1) <= INFIFO_SEL(2) OR INFIFO_SEL(3);
      MUX_SEL(2) <= INFIFO_SEL(4) OR INFIFO_SEL(5);
    end if;
  end process;

  process (MUX_SEL, INFIFO0_D, INFIFO1_D, INFIFO2_D, INFIFO3_D, INFIFO4_D, INFIFO5_D)
  begin
    case MUX_SEL is
      when "000"  => DATA_IN <= INFIFO0_D;
      when "001"  => DATA_IN <= INFIFO1_D;
      when "010"  => DATA_IN <= INFIFO2_D;
      when "011"  => DATA_IN <= INFIFO3_D;
      when "100"  => DATA_IN <= INFIFO4_D;
      when "101"  => DATA_IN <= INFIFO5_D;
      when others => DATA_IN <= (others=>'0');
    end case;
  end process;

  iDAC : CTR_DAC
    port map (
      RST             => RST,
      FCLK            => FCLK,
      SCLK            => SCLK,
      READ_EN         => DAC_DIV,
      WRITE_A         => DATA_IN(16),
      RESET_A         => DATA_IN(17),
      WRITE_B         => DATA_IN(18),
      RESET_B         => DATA_IN(19),
      INFIFO_SEL      => INFIFO_SEL,
      INFIFO_DVALID   => INFIFO_DVALID,
      INFIFO_READ_REQ => INFIFO_READ_REQ,
      WEN_A           => WEN_A,
      WR_ADDR_A       => WR_ADDR_A,
      WEN_B           => WEN_B,
      WR_ADDR_B       => WR_ADDR_B,
      RD_ADDR         => RD_ADDR );

  iDPA : DPRAM
    port map (
      RST     => RST,
      WRCLK   => FCLK,
      WEN     => WEN_A,
      WR_ADDR => WR_ADDR_A,
      DIN     => DATA_IN(15 downto 0),
      --
      RDCLK   => SCLK,
      REN     => DAC_DIV,
      RD_ADDR => RD_ADDR,
      DOUT    => DATA_A );

  iDPB : DPRAM
    port map (
      RST     => RST,
      WRCLK   => FCLK,
      WEN     => WEN_B,
      WR_ADDR => WR_ADDR_B,
      DIN     => DATA_IN(15 downto 0),
      --
      RDCLK   => SCLK,
      REN     => DAC_DIV,
      RD_ADDR => RD_ADDR,
      DOUT    => DATA_B );



  iADD : ddc_add
   port map (
     A     => DATA_A(13 downto 0),
     B     => DATA_B(13 downto 0),
     Q     => ADD_OP,
     CLK   => SCLK);

 DAC_A(13) <= NOT ADD_OP(13);
 DAC_A(12 downto 0) <= ADD_OP(12 downto 0);
 SCLK_DAC_A <= SCLK;   --clock of DAC_A



end RTL;
