-- ==============================================================
-- FILE : USER_AP2.VHD  -  MUST be modified by the USER
-- ==============================================================
--
-- This file is part of Example 2, for the HERON-IO2V2 module.
--
-- This file must be modified by the user.
--
-- ==============================================================
--
--  Date        : 08/02/2002
--  Author      : L. Bacquart - ALSE
--  Description : User code section
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     L. Bacquart    08-02-02    First Written
--  1.1     R. Williams    26-02-02    Changed names, added known
--                                     I/Os, modified IFD and OFD
--                                     implementation.
--  2.0     R. Williams    10-05-02    Adapted for the IO2V2 from
--                                     IO2V1.
--  2.1     R. Williams    10-10-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--  3.1     T. Hollis      11-06-03    Added code for fft-example
--  3.2     R. Williams    08-06-06    Removed HEPC8 support.
--                                     Minor fix to HE_RD_6F logic.
--
-- ==============================================================



-- ==============================================================
--   Package : CONFIG
-- ==============================================================
--
-- This package allow users to set up the different clock domains
-- of the application (FIFO, ADC interfaces) in the top level.
-- Please refer to the HUNT documentation for more details.
--
-- ==============================================================

package CONFIG is

  -- FIFO CLOCK
  -- ==========

--   -----------------------------------------------------------------
--  | FCLK_G_DOMAIN |  HIGH_FCLK_G  |  HIGH_FCLK_RD  |  HIGH_FCLK_WR  |
--   -----------------------------------------------------------------
--  |     True      | True / False  |     n.a.       |     n.a.       |  single FIFO clock
--   -----------------------------------------------------------------
--  |     False     |     n.a.      |  True / False  |  True / False  |  two FIFO clocks
--   -----------------------------------------------------------------

  -- To use a single common FIFO clock, set FCLK_G_DOMAIN = TRUE
  constant FCLK_G_DOMAIN : boolean := TRUE;

  -- When read/write FIFO clock frequency > 60 MHz, set HIGH_FCLK_G = TRUE
  -- This constant is used only when FCLK_G_DOMAIN = TRUE
  constant HIGH_FCLK_G   : boolean := TRUE;

  -- When read FIFO clock frequency > 60 MHz, set HIGH_FCLK_RD = TRUE
  -- This constant is used only when FCLK_G_DOMAIN = FALSE
  constant HIGH_FCLK_RD  : boolean := FALSE;

  -- When write FIFO clock frequency > 60 MHz, set HIGH_FCLK_WR = TRUE
  -- This constant is used only when FCLK_G_DOMAIN = FALSE
  constant HIGH_FCLK_WR  : boolean := FALSE;

  -- ADC CLOCK
  -- =========

  -- To drive ADC sample clock A from a signal inside the FPGA set INTERNAL_SCLK_A = TRUE
  -- to use an external sample clock, set INTERNAL_SCLK_A = FALSE
  constant INTERNAL_SCLK_A : boolean := TRUE;

  -- To use a single common ADC clock for A & B channels, set SCLK_G_DOMAIN = TRUE
  constant SCLK_G_DOMAIN : boolean := TRUE;

end CONFIG;


-- ==============================================================
--   Module : USER_AP
-- ==============================================================

library IEEE;
  use IEEE.std_logic_1164.ALL;
  use IEEE.numeric_std.all;
  use WORK.CONFIG.all;

--pragma translate_off
library sim_lib;
  use sim_lib.all;
--pragma translate_on

entity USER_AP is
  port (
    RESET           : in  std_logic;                     -- asynchronous reset active high
    CONFIG          : in  std_logic;                     -- System wide Config signal
-- CLOCK IO
    OSC0            : in  std_logic;                     -- Xtal OSC0
    OSC1            : in  std_logic;                     -- Xtal OSC1
    OSC2            : in  std_logic;                     -- Xtal OSC2
    OSC3            : in  std_logic;                     -- Xtal OSC3 factory default fitted with 100Mhz
    CLKIN0          : in  std_logic;                     -- User clock input (unbuffered) 
    CLKIN1          : in  std_logic;                     -- User clock input (unbuffered)
    CLKI2           : in  std_logic;                     -- User clock input (buffered)
    CLKI3           : in  std_logic;                     -- User clock input (buffered)
    CLKOUT          : out std_logic;                     -- User clock output
    QTTL            : in  std_logic;
    DTTL            : out std_logic;
    PECL_CLK        : in  std_logic;
-- SERIAL IO
    R1OUT           : in  std_logic;
    R2OUT           : in  std_logic;
    T1IN            : out std_logic;
    T2IN            : out std_logic;
    RS485_232       : out std_logic;
    HDPLX           : out std_logic;
    FAST            : out std_logic;
-- MISCELLANEOUS IO
    UDPRES          : out std_logic;                     -- reset the carrier (high if NOT used)
    LED             : out std_logic_vector(4 downto 0);  -- LEDs
    ADDR_FLAGSEL    : in  std_logic;
    BOOTEN          : out std_logic;
    CID             : in  std_logic_vector(3 downto 0);
    MID             : in  std_logic_vector(3 downto 0);
-- UMIs
    UMI_IN          : in  std_logic_vector(3 downto 0);
    UMI_OUT         : out std_logic_vector(3 downto 0);
    UMI_EN          : out std_logic_vector(3 downto 0);
-- DIGITAL IO
    DIO_IN          : in  std_logic_vector(7 downto 0);
    DIO_OUT         : out std_logic_vector(7 downto 0);
    DIO_EN          : out std_logic_vector(7 downto 0);
-- READ FIFO CLK
    FCLK_RD         : in  std_logic;                     -- FIFO_IN clock to be used in this module (only when FCLK_G_DOMAIN=FALSE)
    SRC_FCLK_RD     : out std_logic;                     -- FIFO_IN clock source for the top level  (only when FCLK_G_DOMAIN=FALSE)
-- WRITE FIFO CLK
    FCLK_WR         : in  std_logic;                     -- FIFO_OUT clock to be used in this module (only when FCLK_G_DOMAIN=FALSE)
    SRC_FCLK_WR     : out std_logic;                     -- FIFO_OUT clock source for the top level  (only when FCLK_G_DOMAIN=FALSE)
-- COMMON FIFO CLK
    FCLK_G          : in  std_logic;                     -- Common FIFO clock to be used in this module (only when FCLK_G_DOMAIN=TRUE)
    SRC_FCLK_G      : out std_logic;                     -- Common FIFO clock source for the top level  (only when FCLK_G_DOMAIN=TRUE)
-- FIFO IN / HE_RD_6F interface
    INFIFO_READ_REQ : out std_logic_vector(5 downto 0);  -- FIFO_IN Read Request
    INFIFO_DVALID   : in  std_logic_vector(5 downto 0);  -- FIFO_IN Data Valid
    INFIFO_SINGLE   : in  std_logic_vector(5 downto 0);  -- FIFO_IN Single data item available
    INFIFO_BURST    : in  std_logic_vector(5 downto 0);  -- FIFO_IN Block of data available
    INFIFO0_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 0
    INFIFO1_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 1
    INFIFO2_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 2
    INFIFO3_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 3
    INFIFO4_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 4
    INFIFO5_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 5
-- FIFO OUT / HE_WR_6F interface
    OUTFIFO_READY   : in  std_logic_vector(5 downto 0);  -- FIFO_OUT Ready Flag
    OUTFIFO_WRITE   : out std_logic_vector(5 downto 0);  -- FIFO_OUT Write Command
    OUTFIFO_D       : out std_logic_vector(31 downto 0); -- FIFO_OUT Data
-- SERIAL BUS / HE_USER Message interface
    MSG_CLK         : out std_logic;                     -- HSB clock source for the top level
    MSG_DIN         : in  std_logic_vector(7 downto 0);  -- data received from HSB
    MSG_ADDR        : in  std_logic_vector(7 downto 0);  -- HSB address
    MSG_AEN         : in  std_logic;                     -- address enable
    MSG_WEN         : in  std_logic;                     -- write access
    MSG_REN         : in  std_logic;                     -- read access
    MSG_DONE        : in  std_logic;                     -- message successfully transmitted
    MSG_COUNT       : in  std_logic;                     -- user's counter enable
    MSG_CLEAR       : in  std_logic;                     -- user's counter clear
    MSG_READY       : out std_logic;                     -- to acknowledge an access
    MSG_SEND_MSG    : out std_logic;                     -- message send command
    MSG_CE          : out std_logic;                     -- to control speed operation
    MSG_DOUT        : out std_logic_vector(7 downto 0);  -- data sent to HSB
    MSG_SEND_ID     : out std_logic;                     -- ID information send command
    MSG_LAST_BYTE   : out std_logic;                     -- to indicate when the last byte is presented
-- ADC_A SCLK
    SCLK_A          : in  std_logic;                     -- ADC A clock to be used in this module (only when SCLK_G_DOMAIN=FALSE)
    SRC_SCLK_A      : out std_logic;                     -- ADC A clock source for the top level  (only when SCLK_G_DOMAIN=FALSE)
-- ADC_B SCLK
    SCLK_B          : in  std_logic;                     -- ADC B clock to be used in this module (only when SCLK_G_DOMAIN=FALSE)
    SRC_SCLK_B      : out std_logic;                     -- ADC B clock source for the top level  (only when SCLK_G_DOMAIN=FALSE)
-- GLOBAL ADC SCLK
    SCLK_G          : in  std_logic;                     -- Common ADC clock to be used in this module (only when SCLK_G_DOMAIN=TRUE)
    SRC_SCLK_G      : out std_logic;                     -- Common ADC clock source for the top level  (only when SCLK_G_DOMAIN=TRUE)
-- HE_ADC_A
    ADC_A           : in  std_logic_vector(11 downto 0); -- ADC_A data
    OTR_A           : in  std_logic;                     -- ADC_A over range flag
-- HE_ADC_B
    ADC_B           : in  std_logic_vector(11 downto 0); -- ADC_B data
    OTR_B           : in  std_logic;                     -- ADC_B over range flag
-- HE_DAC_A
    SCLK_DAC_A      : out std_logic;                     -- DAC_A clock source for the top level
    DAC_A           : out std_logic_vector(13 downto 0); -- DAC_A data
-- HE_DAC_B
    SCLK_DAC_B      : out std_logic;                     -- DAC_B clock source for the top level
    DAC_B           : out std_logic_vector(13 downto 0)  -- DAC_B data
  );
end USER_AP;


architecture EXAMPLE2 of USER_AP is

  component HSB2
    port (
      RST         : in  std_logic;
      CLK         : in  std_logic;
      DIN         : in  std_logic_vector(7 downto 0);
      ADDR        : in  std_logic_vector(7 downto 0);
      AEN         : in  std_logic;
      WEN         : in  std_logic;
      REN         : in  std_logic;
      DONE        : in  std_logic;
      COUNT       : in  std_logic;
      CLEAR       : in  std_logic;
      READY       : out std_logic;
      SEND_MSG    : out std_logic;
      CE          : out std_logic;
      DOUT        : out std_logic_vector(7 downto 0);
      SEND_ID     : out std_logic;
      LAST_BYTE   : out std_logic;
      --
      INFIFO_SEL  : out std_logic_vector (5 downto 0);
      OUTFIFO_SEL : out std_logic_vector (5 downto 0);
      UPD_DONE    : in  std_logic;
      UPDATE      : out std_logic;
      SAMPLE      : out std_logic_vector(9 downto 0);
      IP_SEL      : out std_logic_vector(1 downto 0);
      WIN_SEL     : out std_logic;
      FWD_FFT     : out std_logic;
      DDS_FREQ    : out std_logic_vector(25 downto 0);
      DDS_EN      : out std_logic
    );
  end component;

  component CTR_ADC
    port (
      RST           : in  std_logic;
      SCLK          : in  std_logic;
      FCLK          : in  std_logic;
      UPDATE        : in  std_logic;
      SAMPLE        : in  std_logic_vector(9 downto 0);
      FF_FULL       : in  std_logic;
      FF_EMPTY      : in  std_logic;
      UPD_DONE      : out std_logic;
      FF_WEN        : out std_logic;
      FF_REN        : out std_logic;
      OUTFIFO_SEL   : in  std_logic_vector(5 downto 0);
      OUTFIFO_READY : in  std_logic_vector(5 downto 0);
      OUTFIFO_WRITE : out std_logic_vector(5 downto 0);
      SYNC          : in  std_logic
    );
  end component;

  -- Asynchronous fifo / generated by Xilinx CoreGEN
  component fifo1023x32
    port (
      din    : in  std_logic_vector(31 downto 0);
      wr_en  : in  std_logic;
      wr_clk : in  std_logic;
      rd_en  : in  std_logic;
      rd_clk : in  std_logic;
      ainit  : in  std_logic;
      dout   : out std_logic_vector(31 downto 0);
      full   : out std_logic;
      empty  : out std_logic
    );
  end component;

  component FDC
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic;
      CLR : in std_logic
    );
  end component;

 component FFT
    port(
    RST           : in  std_logic;
    SCLK          : in  std_logic;
    FFT_IP_I      : in  std_logic_vector(15 downto 0);
    FFT_IP_Q      : in  std_logic_vector(15 downto 0);
    FFT_OP_I      : out std_logic_vector(15 downto 0);
    FFT_OP_Q      : out std_logic_vector(15 downto 0);
    SYNC          : out std_logic;
    WIN_SEL       : in  std_logic;
    FWD_FFT       : in std_logic;
    LED0          : out std_logic;
    LED1          : out std_logic;
    RD_MEM_ADDR   : out std_logic_vector(9 downto 0);
    DIO_OUT       :	out std_logic_vector(7 downto 0);
    DIO_EN        :	out std_logic_vector(7 downto 0)
    );
  end component;

 component SIG_SEL
  port (
    RST           : in  std_logic;
    SCLK          : in  std_logic;
    ADC_A         : in  std_logic_vector(11 downto 0);
    MEM_DOUT      : in  std_logic_vector(31 downto 0);
    FFT_IP_I      : out std_logic_vector(15 downto 0);
    FFT_IP_Q      : out std_logic_vector(15 downto 0);
    IP_SEL        : in  std_logic_vector(1 downto 0);
    DDS_FREQ      : in  std_logic_vector(25 downto 0);
    DDS_EN        : in  std_logic
    );
 end component;

 component IP_MEM
  port (
    RST             : in  std_logic;
    FCLK            : in  std_logic;
    SCLK            : in  std_logic;
    INFIFO_SEL      : in  std_logic_vector(5 downto 0);
    INFIFO_DVALID   : in  std_logic_vector(5 downto 0);
    INFIFO_READ_REQ : out std_logic_vector(5 downto 0);
    DATA_IN         : in  std_logic_vector(31 downto 0);
    RD_MEM_ADDR     : in  std_logic_vector(9 downto 0);
    MEM_DOUT        : out std_logic_vector(31 downto 0)
    );
end component;

 component mult
  port (
    clk          : in  std_logic;
    a            : in  std_logic_vector(15 downto 0);
    b            : in  std_logic_vector(15 downto 0);
    q            : out std_logic_vector(15 downto 0)
    );
 end component;

 component adder
  port (
    A            : in  std_logic_VECTOR(15 downto 0);
    B            : in  std_logic_VECTOR(15 downto 0);
    Q            : out std_logic_VECTOR(15 downto 0);
    CLK          : in  std_logic
    );
 end component;


  signal VCC         : std_logic;
  signal MCLK        : std_logic;  -- Main clock
  signal LEDcount    : unsigned(25 downto 0);

  signal INFIFO_SEL  : std_logic_vector(5 downto 0);
  signal OUTFIFO_SEL : std_logic_vector(5 downto 0);
  signal DATA_IN     : std_logic_vector(31 downto 0);
  signal MUX_SEL     : std_logic_vector(2 downto 0);
  signal UPDATE      : std_logic;
  signal SAMPLE      : std_logic_vector(9 downto 0);
  signal UPD_DONE    : std_logic;
  signal FF_FULL     : std_logic;
  signal FF_EMPTY    : std_logic;
  signal FF_WEN      : std_logic;
  signal FF_REN      : std_logic;
  signal ADC_DIN     : std_logic_vector(31 downto 0);

  signal SYNC        : std_logic;
  signal FFT_IP_I    : std_logic_vector(15 downto 0);
  signal FFT_IP_Q    : std_logic_vector(15 downto 0);
  signal FFT_OP_I    : std_logic_vector(15 downto 0);
  signal FFT_OP_Q    : std_logic_vector(15 downto 0);

  signal IP_SEL      : std_logic_vector(1 downto 0);
  signal WIN_SEL     : std_logic;
  signal FWD_FFT     : std_logic;
  signal DDS_FREQ    : std_logic_vector(25 downto 0);
  signal DDS_EN      : std_logic;

  signal ADD_OP      : std_logic_vector(15 downto 0);
  signal MULT_OP_I   : std_logic_vector(15 downto 0);
  signal MULT_OP_Q   : std_logic_vector(15 downto 0);

  signal RD_MEM_ADDR : std_logic_vector(9 downto 0);
  signal MEM_DOUT    : std_logic_vector(31 downto 0);

  signal MULT_I      : std_logic_vector(15 downto 0);
  signal MULT_Q      : std_logic_vector(15 downto 0);


begin


  -- ==========================================================
  -- ==========================================================
  --       <<<<<<<   INSERT YOUR CODE HERE    >>>>>>>>>
  -- ==========================================================
  -- ==========================================================

  VCC <= '1';

  -------------------------------------------------------------
  -- OSC3 clock

  MCLK <= OSC3;

  -------------------------------------------------------------
  -- Drive the UDP-RESET high (inactive)

  UDPRES <= '1';

  -------------------------------------------------------------
  -- Drive the BOOT ENABLE signal high (inactive)

  BOOTEN <= '1';

  -------------------------------------------------------------
  -- Drive unused signals

  UMI_OUT   <= (others=>'0');
  UMI_EN    <= (others=>'1');

  CLKOUT    <= '0';

  DTTL      <= '0';
  T1IN      <= '0';
  T2IN      <= '0';
  RS485_232 <= '0';
  HDPLX     <= '0';
  FAST      <= '0';

  -------------------------------------------------------------
  -- IN/OUT FIFO clocks

  SRC_FCLK_RD <= '0';   -- clock source for READ FIFO
  SRC_FCLK_WR <= '0';   -- clock source for WRITE FIFO
  SRC_FCLK_G  <= MCLK;  -- global clock source for both FIFOs

  -------------------------------------------------------------
  -- Serial Bus

  MSG_CLK <= OSC3;  -- clock for Message Interface

  iHSB2 : HSB2
    port map (
      RST         => RESET,
      CLK         => OSC3,
      DIN         => MSG_DIN,
      ADDR        => MSG_ADDR,
      AEN         => MSG_AEN,
      WEN         => MSG_WEN,
      REN         => MSG_REN,
      DONE        => MSG_DONE,
      COUNT       => MSG_COUNT,
      CLEAR       => MSG_CLEAR,
      READY       => MSG_READY,
      SEND_MSG    => MSG_SEND_MSG,
      CE          => MSG_CE,
      DOUT        => MSG_DOUT,
      SEND_ID     => MSG_SEND_ID,
      LAST_BYTE   => MSG_LAST_BYTE,
      INFIFO_SEL  => INFIFO_SEL,
      OUTFIFO_SEL => OUTFIFO_SEL,
      UPD_DONE    => UPD_DONE,
      UPDATE      => UPDATE,
      SAMPLE      => SAMPLE,
      IP_SEL      => IP_SEL,
      WIN_SEL     => WIN_SEL,
	 FWD_FFT     => FWD_FFT,
      DDS_FREQ    => DDS_FREQ,
      DDS_EN      => DDS_EN );

  -------------------------------------------------------------
  -- ADC
  SRC_SCLK_A <= '0';   -- clock source for ADC_A
  SRC_SCLK_B <= '0';   -- clock source for ADC_B
  SRC_SCLK_G <= OSC3;  -- global clock source for both ADCs


-- HERON INTERFACE OUTPUT
  ADC_DIN(15 downto 0)  <= FFT_OP_I; -- PHASE IN LS HALF WORD
  ADC_DIN(31 downto 16) <= FFT_OP_Q; -- QUAD  IN MS HALF WORD


  iADC : CTR_ADC
    port map (
      RST           => RESET,
      SCLK          => SCLK_G,
      FCLK          => FCLK_G,
      UPDATE        => UPDATE,
      SAMPLE        => SAMPLE,
      FF_FULL       => FF_FULL,
      FF_EMPTY      => FF_EMPTY,
      UPD_DONE      => UPD_DONE,
      FF_WEN        => FF_WEN,
      FF_REN        => FF_REN,
      OUTFIFO_SEL   => OUTFIFO_SEL,
      OUTFIFO_READY => OUTFIFO_READY,
      OUTFIFO_WRITE => OUTFIFO_WRITE,
      SYNC          => SYNC );

  ififo1023x32 : fifo1023x32
    port map (
      din    => ADC_DIN,
      wr_en  => FF_WEN,
      wr_clk => SCLK_G,
      rd_en  => FF_REN,
      rd_clk => FCLK_G,
      ainit  => RESET,
      dout   => OUTFIFO_D,
      full   => FF_FULL,
      empty  => FF_EMPTY );

  -------------------------------------------------------------
  -- DAC

  SCLK_DAC_A <= SCLK_G;  -- clock of DAC_A
  SCLK_DAC_B <= SCLK_G;  -- clock of DAC_B

  -- FFT MAGNITUDE SQUARED OUTPUT ON DAC-A
  -- Extra gain of x8 on default mode
    process(ADD_OP)
  begin
     if((IP_SEL = "00") and (WIN_SEL = '0')) then
       DAC_A(13)          <= not ADD_OP(15);
       DAC_A(12 downto 0) <=     ADD_OP(9 downto 0) & "000";
     else
       DAC_A(13)          <= not ADD_OP(15);
       DAC_A(12 downto 0) <=     ADD_OP(12 downto 0);
     end if;
  end process;

  -- FFT PHASE INPUT ON DAC-B
  DAC_B(13)          <= not FFT_IP_I(15);
  DAC_B(12 downto 0) <=     FFT_IP_I(14 downto 2);

  -------------------------------------------------------------
    process(FCLK_G)
  begin
    if rising_edge(FCLK_G) then
      MUX_SEL(0) <= INFIFO_SEL(1) OR INFIFO_SEL(3) OR INFIFO_SEL(5);
      MUX_SEL(1) <= INFIFO_SEL(2) OR INFIFO_SEL(3);
      MUX_SEL(2) <= INFIFO_SEL(4) OR INFIFO_SEL(5);
    end if;
  end process;

  process (MUX_SEL, INFIFO0_D, INFIFO1_D, INFIFO2_D, INFIFO3_D, INFIFO4_D, INFIFO5_D)
  begin
    case MUX_SEL is
      when "000"  => DATA_IN <= INFIFO0_D;
      when "001"  => DATA_IN <= INFIFO1_D;
      when "010"  => DATA_IN <= INFIFO2_D;
      when "011"  => DATA_IN <= INFIFO3_D;
      when "100"  => DATA_IN <= INFIFO4_D;
      when "101"  => DATA_IN <= INFIFO5_D;
      when others => DATA_IN <= (others=>'0');
    end case;
  end process;

      iMEM : IP_MEM
    port map (
      RST             => RESET,
      FCLK            => FCLK_G,
      SCLK            => SCLK_G,
      INFIFO_SEL      => INFIFO_SEL,
      INFIFO_DVALID   => INFIFO_DVALID,
      INFIFO_READ_REQ => INFIFO_READ_REQ,
      DATA_IN         => DATA_IN,
      RD_MEM_ADDR     => RD_MEM_ADDR,
	 MEM_DOUT        => MEM_DOUT );


 -------------------------------------------------------------
 -- FFT
   iFFT : FFT
    port map (
      RST         => RESET,
      SCLK        => SCLK_G,
      FFT_IP_I    => FFT_IP_I,
      FFT_IP_Q    => FFT_IP_Q,
      FFT_OP_I    => FFT_OP_I,
      FFT_OP_Q    => FFT_OP_Q,
      SYNC        => SYNC,
      WIN_SEL     => WIN_SEL,
	 FWD_FFT     => FWD_FFT,
      LED0        => LED(0),
      LED1        => LED(1),
      RD_MEM_ADDR => RD_MEM_ADDR,
	 DIO_OUT     => DIO_OUT,
      DIO_EN      => DIO_EN );

-- INPUT SIGNAL SOURCE SELECT
  iSIG_SEL : SIG_SEL
   port map (
     RST      => RESET,
     SCLK     => SCLK_G,
     ADC_A    => ADC_A,
     MEM_DOUT => MEM_DOUT,
     FFT_IP_I => FFT_IP_I,
     FFT_IP_Q => FFT_IP_Q,
     IP_SEL   => IP_SEL,
     DDS_FREQ => DDS_FREQ,
     DDS_EN   => DDS_EN );

 --CALCULATE FFT OUTPUT MAGNITUDE SQUARED
    MULT_I(15)          <= FFT_OP_I(15);
    MULT_I(14 downto 0) <= FFT_OP_I(14 downto 0);

  iMULT_I : mult
   port map (
     clk      => SCLK_G,
     a        => MULT_I,
     b        => MULT_I,
     q        => MULT_OP_I);

    MULT_Q(15)          <= FFT_OP_Q(15);
    MULT_Q(14 downto 0) <= FFT_OP_Q(14 downto 0);

  iMULT_Q : mult
   port map (
     clk      => SCLK_G,
     a        => MULT_Q,
     b        => MULT_Q,
     q        => MULT_OP_Q);

  iADDER : adder
   port map (
     A        => MULT_OP_I,
     B        => MULT_OP_Q,
     Q        => ADD_OP,
     CLK      => SCLK_G );

  -------------------------------------------------------------
  -- Led Flasher : dividing F_CLKG clock by 2**25

  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      LEDcount <= (others=>'0');
    elsif rising_edge(FCLK_G) then
      LEDcount <= LEDcount + 1;
    end if;
  end process;

  LED(4) <= LEDcount(24);

  -- Drive LEDs high (inactive)
  LED(3) <= '1';
  LED(2) <= '1';

end EXAMPLE2;

